
#include "GPumpSettingsData.h"

G_DEFINE_TYPE (GPumpSettingsData, gpump_settings_data, G_TYPE_OBJECT)


/* Private datastructure */
struct GPumpSettingsData {
#if GPUMP_USE_GSETTINGS
  /* org.gego.gpump.ui */
  GSettings * ui_settings;
  /* org.gego.gpump */
  GSettings * gpump_settings;
#else
  GKeyFile * gpump_settings; /* Stores both UI and other settings... */
  GFile * gpump_settigs_file;
#endif /* GPUMP_USE_GSETTINGS */
}

/* * Override the default constructer *****************************************/
GObject * _constructor (GType type,
                        guint n_construct_params,
                        GObjectConstructParam * construct_params);


/******************************************************************************/
GPumpSettingsData * gpump_settings_data_get_default () {
  GPumpSettingsData * self = g_object_new (GPUMP_TYPE_SETTINGS_DATA,
                                           NULL);
  
  return self;
}

void gpump_settings_data_init (GPumpSettingsData * self) {
  g_print ("Init GPumpSettingsData\n");
  
  self->priv = GPUMP_SETTINGS_DATA_GET_PRIVATE (self);
  
#if GPUMP_USE_GSETTINGS
  /* This will not be compiled, this will have to be implemented later...
   * much later...
   */
  self->ui_settings = g_settings_new ("org.gego.gpump.ui");
  self->gpump_settings = g_settings_new ("org.gego.gpump");
#else
  
  /* Construct the path for the files, 1) the GPump settings 2) the GPump UI 
   * settings.
   *
   * The file names are defined static, so they do not need to be re-created,
   * each and every time te object is re-created.
   */
  
  self->priv->gpump_settings = g_key_file_new ();
  
  static gchar * gpump_settings_file_uri = NULL;
  
  if (!(gpump_settings_file_uri)) {
    
    gchar * base_config_dir = g_get_user_config_dir ();
    
    gchar * settings_basedir = g_strdup_printf ("%s/%s",
                                                base_config_dir,
                                                "gpump");
    g_free (base_config_dir);
    
    gpump_settings_file_uri = g_strdup_printf ("%s/%s",
                                               settings_basedir,
                                               "gpump.conf");
  }
  
  /* Do not want to re-create this object every time, now do we? */
  self->priv->gpump_settings_file = g_file_new_from_path (gpump_settings_uri);
  
  GError * err = NULL;
  if (!( g_key_file_load_from_file (self->priv->gpump_settings,
                                    pump_settings_file_uri,
                                    G_KEY_FILE_NONE,
                                    err)) {
    /* File does not exist or something else might be wrong. */
    if (err) {
      /* Did the file exist? */
      if (!(g_error_match(err, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_NOT_FOUND))) {
        g_error ("GPumpSettingsData:\nAn %s error occured:\n\t%s\n\n",
                 g_quark_to_string (err->domain),
                 err->message);
        g_clear_error (err);
      } else { 
        /* The file did not exist, thus we must create a key-file and store it.
         */
        g_clear_error (err);
        GOutputStream * out_stream = g_file_create (
                                               self->priv->gpump_settings_file,
                                               G_FILE_CREATE_PRIVATE,
                                               NULL,
                                               err);
        g_object_unref (out_stream);
        
        if (err) {
          g_error ("GPumpSettingsData: %s\n\t%s\n\n",
                   g_quark_to_string (err->domain),
                   err->messarge);
          g_clear_error (err);
        }
        
        /* UI settings */
        /* -1 is undefined, these are set when the main window is displayed */
        g_key_file_set_integer (self->priv->gpump_settings, "ui","x", -1);
        g_key_file_set_integer (self->priv->gpump_settings, "ui","y", -1);
        /* 500x500 is the default window size */
        g_key_file_set_integer (self->priv->gpump_settings, "ui","h", 500);
        g_key_file_set_integer (self->priv->gpump_settings, "ui","x", 500);
        /* main settings */
        g_key_file_set_boolean (self->priv->gpump_settings,
                                "main",
                                "use dark",
                                FALSE);
        
        /* commit data to file. */
        if (!(gpump_settings_data_commit (self, err))) {
          g_error ("GPumpSettingsData:\n An %s error occured:\n\t%s\n\n",
                   g_quark_to_string (err->domain),
                   err->messarge);
        }
      }
    }
  }
#endif /* GPUMP_USE_GSETTINGS */
}


void gpump_settings_data_class_init (GPumpSettingsDataClass * klass) {
  GObjectClass * parent = G_OBJECT_CLASS (klass);
  
  parent->constructor = _constructor;
  
  g_type_class_add_private (klass, sizeof (GPumpSettingsDataPrivate);
}


void gpump_settings_data_set (GPumpSettingsData * self,
                              gchar * name,
                              GVariant * data) {
#if GPUMP_USE_GSETTINGS
  /* TODO */
#else
  
#endif
}


GVariant * gpump_settings_data_get (GPumpSettingsData * self,
                                    gchar * name) {
#if GPUMP_USE_GSETTINGS
  /* TODO */
#else
  
#endif
}


gboolean gpump_settings_data_commit (GPumpSettingsData * self,
                                     GError * err) {
#if GPUMP_USE_GSETTINGS
  /* Do thothing */
  return TRUE;
#else
  g_assert (err == NULL);
  
  GFileOutputStream * out_stream = g_file_replace (
                                                self->priv->gpump_settings_file,
                                                NULL,
                                                FALSE,
                                                G_FILE_CREATE_PRIVATE,
                                                NULL,
                                                err);
  if (err) {
    g_object_unref (out_stream);
    return FALSE;
  }
  
  gsize * len = NULL;
  gchar * buff = g_key_file_to_data (self->priv->gpump_settings, len, err);
  
  if (err) {
   g_object_unref (out_stream);
   g_free (len);
   g_free (buff);
   return FALSE;
  }
  
  g_output_stream_write (out_stream,
                         buff,
                         len,
                         NULL,
                         err);
  
  if (err) {
    g_object_unref (out_stream);
    g_free (len);
    g_free (buff);
    return FALSE;
  }
  
  g_object_unref (out_stream);
  g_free (len);
  g_free (buff);
  
  return TRUE;
#endif
}


/* * Implementation of the constructor ****************************************
 *
 * See: blogs.gnome.org/xclaesse/2010/02/11/how-to-make-a-gobject-singleton/
 * for more information.
 */
GObject * _constructor (GType type,
                        guint n_construct_params,
                        GObjectConstructParam * construct_params) {
  static GObject * self = NULL;
  static GMutex mutex;
  
  g_mutex_lock (&mutex);
  if (self == NULL) {
    self = G_OBJECT_CLASS (gpump_settings_data_parent_class)->constructor (
      type, n_construct_params, construct_params);
    g_object_add_weak_pointer (self, (gpointer) &self);
    g_mutex_unlock (&mutex);
    return self;
  }
  g_mutex_unlock (&mutex);
  return g_object_ref (self);
}


